# NewRewards release 4 by David Bowland
# ./addons/eventscripts/newrewards/newrewards.py

# >>> To configure this addon please see newrewards.cfg <<<

"""
Changes the money and score received for certain actions. Requires only ES 2.0+
"""


import es
import os.path


info = es.AddonInfo()
info.name = 'NewRewards'
info.version = '4'
info.url = 'http://addons.eventscripts.com/addons/view/newrewards'
info.basename = 'newrewards'
info.author = 'SuperDave'


str_last_winner = ''
dict_team_money = {}
list_found_hostages = []
dict_players = {}

dict_round_end_reasons = {'1':{'2':'newrewards_teamwin_accomplishgoal', '3':'newrewards_teamloss_failtopreventgoal'}, '7':{'2':'newrewards_teamloss_goalnotaccomplished', '3':'newrewards_teamwin_preventgoal'}, '8':{'2':'newrewards_teamloss_eliminated', '3':'newrewards_teamwin_eliminateopponents'}, '9':{'2':'newrewards_teamwin_eliminateopponents', '3':'newrewards_teamloss_eliminated'}, '11':{'2':'newrewards_teamloss_failtopreventgoal', '3':'newrewards_teamwin_accomplishgoal'}, '12':{'2':'newrewards_teamloss_goalnotaccomplished', '3':'newrewards_teamwin_preventgoal'}, '13':{'2':'newrewards_teamwin_preventgoal', '3':'newrewards_teamloss_goalnotaccomplished'}}

dict_options = {'newrewards_spawnmoney_control':[1, '0 = no change, 1 = set player money when players spawn'], 'newrewards_startmoney':[1600, 'Amount of money a player will have when he or she starts the game--normal Counter-Strike value 800'], 'newrewards_teamwin_min':[1700, 'Minimum amount a team will receive after winning a round'], 'newrewards_teamwin_eliminateopponents':[500, 'Amount a team will receive for each round the opposing team was eliminated--count resets after a loss'], 'newrewards_teamwin_accomplishgoal':[700, 'Amount a team will receive for each round the goal was accomplished--count resets after a loss'], 'newrewards_teamwin_preventgoal':[400, 'Amount a team will receive for each round the goal was prevented--count resets after a loss'], 'newrewards_teamloss_min':[1200, 'Minimum amount a team will receive after losing a round'], 'newrewards_teamloss_eliminated':[600, 'Amount a team will receive for each round they were eliminated--count resets after a win'], 'newrewards_teamloss_goalnotaccomplished':[900, 'Amount a team will receive for each round they did not accomplish the goal--count resets after a win'], 'newrewards_teamloss_failtopreventgoal':[700, 'Amount a team will receive for each round they failed to prevent the goal from being accomplished--count resets after a win'], 'newrewards_gameplaymoney_control':[1, '0 = no change, 1 = set player money for kill, death, and objective events'], 'newrewards_money_kill_default':[300, 'Normal kills--normal Counter-Strike value 300', 300], 'newrewards_money_kill_headshot':[450, 'Headshot kills--normal Counter-Strike value 300', 300], 'newrewards_money_kill_knife':[600, 'Knife kills--normal Counter-Strike value 300', 300], 'newrewards_money_kill_hegrenade':[500, 'HE grenade kills--normal Counter-Strike value 300', 300], 'newrewards_money_kill_team':[-2000, 'Team kills--normal Counter-Strike value -3300', -3300], 'newrewards_money_death_default':[0, 'Normal deaths--normal Counter-Strike value 0', 0], 'newrewards_money_death_headshot':[-100, 'Headshot deaths--normal Counter-Strike value 0', 0], 'newrewards_money_death_knife':[-300, 'Knife deaths--normal Counter-Strike value 0', 0], 'newrewards_money_death_hegrenade':[-200, 'HE grenade deaths--normal Counter-Strike value 0', 0], 'newrewards_money_death_team':[0, 'Team deaths--normal Counter-Strike value 0', 0], 'newrewards_money_hostage_follow':[150, 'First player to make a hostage follow (per hostage)--normal Counter-Strike value 150', 150], 'newrewards_money_hostage_rescue':[500, 'Hostage rescue--normal Counter-Strike value 1000', 1000], 'newrewards_money_bomb_plant':[300, 'Bomb plant--normal Counter-Strike value 0', 0], 'newrewards_money_bomb_explode':[1200, 'Bomb plant that explodes--normal Counter-Strike value 0', 0], 'newrewards_money_bomb_defuse':[500, 'Bomb defuse--normal Counter-Strike value 0', 0], 'newrewards_gameplayscore_control':[1, '0 = no change, 1 = set player score for kill, death, and objective events'],  'newrewards_score_kill_default':[1, 'Normal kills--normal Counter-Strike value 1', 1], 'newrewards_score_kill_headshot':[1.5, 'Headshot kills--normal Counter-Strike value 1', 1], 'newrewards_score_kill_knife':[3, 'Knife kills--normal Counter-Strike value 1', 1], 'newrewards_score_kill_hegrenade':[2, 'HE grenade kills--normal Counter-Strike value 1', 1], 'newrewards_score_kill_team':[-2, 'Team kills--normal Counter-Strike value -1', -1], 'newrewards_score_death_default':[0, 'Normal deaths--normal Counter-Strike value 0', 0], 'newrewards_score_death_headshot':[-0.5, 'Headshot deaths--normal Counter-Strike value 0', 0], 'newrewards_score_death_knife':[-1, 'Knife deaths--normal Counter-Strike value 0', 0], 'newrewards_score_death_hegrenade':[-0.5, 'HE grenade deaths--normal Counter-Strike value 0', 0], 'newrewards_score_death_team':[0, 'Team deaths--normal Counter-Strike value 0', 0], 'newrewards_score_hostage_follow':[0, 'First player to make a hostage follow (per hostage)--normal Counter-Strike value 0', 0], 'newrewards_score_hostage_rescue':[0.3, 'Hostage rescue--normal Counter-Strike value 0', 0], 'newrewards_score_bomb_plant':[1, 'Bomb plant--normal Counter-Strike value 0', 0], 'newrewards_score_bomb_explode':[2, 'Bomb plant that explodes--normal Counter-Strike value 3', 3], 'newrewards_score_bomb_defuse':[2, 'Bomb defuse--normal Counter-Strike value 3 ', 3]}


def load():
   """
   Checks to make sure critical server variables are created by the config
   Calls es_map_start
   Calls player_spawn for each player for initialization
   """

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('newrewards') + '/newrewards.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/newrewards/newrewards.cfg')
   else:
      es.dbgmsg(0, 'NewRewards: Unable to load newrewards.cfg! Please ensure it is in the ./newrewards/ directory.')

   es_map_start({})

   dict_player_list = es.createplayerlist()
   for int_userid in dict_player_list:
      player_spawn({'userid':str(int_userid), 'es_userteam':str(dict_player_list[int_userid]['teamid']), 'es_userkills':str(dict_player_list[int_userid]['kills'])})


def es_map_start(event_var):
   """Resets team and player values to fresh values"""
   global str_last_winner
   global dict_team_money

   str_last_winner = ''
   dict_team_money = {'2':0, '3':0}
   dict_players.clear()


def round_start(event_var):
   """Clears the list of hostages told to follow for the current round"""
   global list_found_hostages

   list_found_hostages[:] = []


def round_end(event_var):
   """
   Adds money to team amounts
   Stores the amount of money each player has for an accurate amount to use in player_spawn
   """
   global str_last_winner
   global dict_team_money
   global dict_players

   str_reason = event_var['reason']
   if str_reason == '16':
      es_map_start({})
   elif dict_round_end_reasons.has_key(str_reason):
      if str_reason in ('1', '9', '13') and str_last_winner <> '2':
         dict_team_money = {'2':int(es.ServerVar('newrewards_teamwin_min')), '3':int(es.ServerVar('newrewards_teamloss_min'))}
         str_last_winner = '2'
      elif str_reason in ('7', '8', '11', '12') and str_last_winner <> '3':
         dict_team_money = {'2':int(es.ServerVar('newrewards_teamloss_min')), '3':int(es.ServerVar('newrewards_teamwin_min'))}
         str_last_winner = '3'

      dict_var_names = dict_round_end_reasons[str_reason]
      for str_team in ('2', '3'):
         dict_team_money[str_team] += int(es.ServerVar(dict_var_names[str_team]))

      for str_userid in dict_players:
         dict_players[str_userid]['round_end'] = es.getplayerprop(str_userid, 'CCSPlayer.m_iAccount')


def player_spawn(event_var):
   """
   Creates the spawning player in the dictionary
   Calculates player spawn money
   Sets player spawn money
   """
   global dict_players

   str_team = event_var['es_userteam']
   if int(str_team) > 1:
      str_userid = event_var['userid']
      if not dict_players.has_key(str_userid):
         dict_players[str_userid] = {}
         dict_players[str_userid]['score'] = int(event_var['es_userkills'])

      dict_current_player = dict_players[str_userid]
      if dict_current_player.has_key('round_end'):
         int_money = dict_current_player['round_end']
         del dict_current_player['round_end']
      else:
         int_money = int(es.ServerVar('newrewards_startmoney'))

      if int(es.ServerVar('newrewards_spawnmoney_control')):
         int_money = check_range(int_money + dict_team_money[str_team])
         es.setplayerprop(str_userid, 'CCSPlayer.m_iAccount', int_money)
         dict_current_player['money'] = int_money
      else:
         dict_current_player['money'] = es.getplayerprop(str_userid, 'CCSPlayer.m_iAccount')


def player_death(event_var):
   """
   Determines the type of event this death was
   Triggers the reward event for the victim and attacker
   """
   str_attacker = event_var['attacker']
   str_weapon = event_var['weapon']

   if event_var['es_userteam'] == event_var['es_attackerteam']:
      str_reward = 'team'
   elif str_weapon == 'hegrenade':
      str_reward = 'hegrenade'
   elif str_weapon == 'knife':
      str_reward = 'knife'
   elif int(event_var['headshot']):
      str_reward = 'headshot'
   else:
      str_reward = 'default'

   trigger_reward(event_var['userid'], 'death_' + str_reward)
   if int(str_attacker):
      trigger_reward(str_attacker, 'kill_' + str_reward)


def player_disconnect(event_var):
   """Removes the player from the dictionary"""
   global dict_players

   str_userid = event_var['userid']
   if dict_players.has_key(str_userid):
      del dict_players[str_userid]


def hostage_follows(event_var):
   """Triggers the hostage_follow reward event if the hostage has not been told to follow this round"""
   global list_found_hostages

   str_hostage = event_var['hostage']
   if str_hostage not in list_found_hostages:
      list_found_hostages.append(str_hostage)
      trigger_reward(event_var['userid'], 'hostage_follow')


def hostage_rescued(event_var):
   """Triggers the hostage_rescue reward event"""
   trigger_reward(event_var['userid'], 'hostage_rescue')


def bomb_planted(event_var):
   """Triggers the bomb_plant reward event"""
   trigger_reward(event_var['userid'], 'bomb_plant')


def bomb_defused(event_var):
   """Triggers the bomb_defuse reward event"""
   trigger_reward(event_var['userid'], 'bomb_defuse')


def bomb_exploded(event_var):
   """Triggers the bomb_explode reward event"""
   trigger_reward(event_var['userid'], 'bomb_explode')


def unload():
   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def trigger_reward(str_userid, str_event):
   """
   Updates player money based on the reward event
   Updates player score based on the reward event
   """
   global dict_players

   dict_current_player = dict_players[str_userid]

   if int(es.ServerVar('newrewards_gameplaymoney_control')):
      int_money = es.getplayerprop(str_userid, 'CCSPlayer.m_iAccount')
      int_previous_money = dict_current_player['money']
      int_money_change = dict_options['newrewards_money_' + str_event][2]

      if not int_money and int_previous_money + int_money_change < 0:
         int_money = int_previous_money + int_money_change
      if int_money == 16000 and int_previous_money + int_money_change > 16000:
         int_money = int_previous_money + int_money_change

      int_money = check_range(int_money + int(es.ServerVar('newrewards_money_' + str_event)) - int_money_change)
      es.setplayerprop(str_userid, 'CCSPlayer.m_iAccount', int_money)
      dict_current_player['money'] = int_money

   if int(es.ServerVar('newrewards_gameplayscore_control')):
      int_old_score = es.createplayerlist(str_userid)[int(str_userid)]['kills']
      if int_old_score <> int(dict_current_player['score']):
         dict_current_player['score'] += float(int_old_score - int(dict_current_player['score']))
      dict_current_player['score'] += float(es.ServerVar('newrewards_score_' + str_event)) - dict_options['newrewards_score_' + str_event][2]
      if int(dict_current_player['score']) <> int_old_score:
         dict_game_scores = es.createentitylist('game_score')
         int_game_score_index = es.getentityindex('game_score')
         if len(dict_game_scores) > 1:
            for int_index in dict_game_scores:
               if int_index <> int_game_score_index:
                  es.server.cmd('es_xremove %s' % int_index)
         elif int_game_score_index == -1:
            es.server.cmd('es_xgive %s game_score' % str_userid)
         es.server.cmd('es_xfire %s game_score addoutput \"spawnflags 1\"' % str_userid)
         es.server.cmd('es_xfire %s game_score addoutput \"points %s\"' % (str_userid, int(dict_current_player['score']) - int_old_score))
         es.server.cmd('es_xfire %s game_score applyscore' % str_userid)


def check_range(int_money):
   """Returns int_money within the desired range"""

   if int_money > 16000:
      int_money = 16000
   elif int_money < 0:
      int_money = 0

   return int_money